/*
 * Copyright (c) 2007 Hewlett-Packard Development Company, L.P.
 * Contributed by Stephane Eranian <eranian@hpl.hp.com>
 *
 * This file should never be included directly, use
 * <perfmon/perfmon.h> instead.
 */

#ifndef _PERFMON_MIPS64_H_
#define _PERFMON_MIPS64_H_

#define PFM_ARCH_MAX_PMCS	(256+64) /* 256 HW 64 SW */
#define PFM_ARCH_MAX_PMDS	(256+64) /* 256 HW 64 SW */

#endif /* _PERFMON_MIPS64_H_ */
