/****************************/
/* THIS IS OPEN SOURCE CODE */
/****************************/

/* 
* File:    map-k7.h
* CVS:     $Id$
* Author:  Harald Servat
*          redcrash@gmail.com
*/

#ifndef FreeBSD_MAP_K7
#define FreeBSD_MAP_K7


enum NativeEvent_Value_K7Processor {
	PNE_K7_DC_ACCESSES = PAPI_NATIVE_MASK,
	PNE_K7_DC_MISSES,
	PNE_K7_DC_REFILLS_FROM_L2,
	PNE_K7_DC_REFILLS_FROM_SYSTEM,
	PNE_K7_DC_WRITEBACKS,
	PNE_K7_L1_DTLB_MISS_AND_L2_DTLB_HITS,
	PNE_K7_L1_AND_L2_DTLB_MISSES,
	PNE_K7_MISALIGNED_REFERENCES,
	PNE_K7_IC_FETCHES,
	PNE_K7_IC_MISSES,
	PNE_K7_L1_ITLB_MISSES,
	PNE_K7_L1_AND_L2_ITLB_MISSES,
	PNE_K7_RETIRED_INSTRUCTIONS,
	PNE_K7_RETIRED_OPS,
	PNE_K7_RETIRED_BRANCHES,
	PNE_K7_RETIRED_BRANCHES_MISPREDICTED,
	PNE_K7_RETIRED_TAKEN_BRANCHES,
	PNE_K7_RETIRED_TAKEN_BRANCHES_MISPREDICTED,
	PNE_K7_RETIRED_FAR_CONTROL_TRANSFERS,
	PNE_K7_RETIRED_RESYNC_BRANCHES,
	PNE_K7_INTERRUPTS_MASKED_CYCLES,
	PNE_K7_INTERRUPTS_MASKED_WHILE_PENDING_CYCLES,
	PNE_K7_HARDWARE_INTERRUPTS,
	/* Nearly special counters */
	PNE_K7_DC_REFILLS_FROM_L2_M,
	PNE_K7_DC_REFILLS_FROM_L2_OES,
	PNE_K7_DC_REFILLS_FROM_SYSTEM_M,
	PNE_K7_DC_REFILLS_FROM_SYSTEM_OES,
	PNE_K7_NATNAME_GUARD
};

extern Native_Event_LabelDescription_t K7Processor_info[];
extern hwi_search_t K7Processor_map[];

#endif
